from unittest.mock import MagicMock, mock_open, patch

import pytest
from mcp.shared.exceptions import McpError


class TestGetPendingTodos:
    @patch("subprocess.run")
    @patch("mcp_tdo.tdo_client.TdoClient._read_file_contents")
    def test_get_pending_todos_with_results(
        self, mock_read_contents, mock_subprocess, tdo_server
    ):
        process_mock = MagicMock()
        process_mock.stdout = "/path/to/todo1.md\n/path/to/todo2.md\n"
        process_mock.returncode = 0
        mock_subprocess.return_value = process_mock

        def mock_file_content(file_path):
            if file_path == "/path/to/todo1.md":
                return (
                    "# Todo 1\n- [ ] Task 1\n- [x] Completed task\n- [ ] Task 2"
                )
            if file_path == "/path/to/todo2.md":
                return "# Todo 2\n- [ ] Another task\n- [ ] Yet another task"
            return ""

        mock_read_contents.side_effect = mock_file_content

        result = tdo_server.get_pending_todos()

        mock_subprocess.assert_called_once_with(
            ["tdo", "t"], capture_output=True, text=True, check=True
        )
        assert len(result.todos) == 4
        assert result.todos[0] == {
            "file": "/path/to/todo1.md",
            "todo": "- [ ] Task 1",
        }
        assert result.todos[1] == {
            "file": "/path/to/todo1.md",
            "todo": "- [ ] Task 2",
        }
        assert result.todos[2] == {
            "file": "/path/to/todo2.md",
            "todo": "- [ ] Another task",
        }
        assert result.todos[3] == {
            "file": "/path/to/todo2.md",
            "todo": "- [ ] Yet another task",
        }

    @patch("subprocess.run")
    def test_get_pending_todos_no_results(self, mock_subprocess, tdo_server):
        process_mock = MagicMock()
        process_mock.stdout = ""
        process_mock.returncode = 0
        mock_subprocess.return_value = process_mock

        result = tdo_server.get_pending_todos()

        mock_subprocess.assert_called_once_with(
            ["tdo", "t"], capture_output=True, text=True, check=True
        )
        assert len(result.todos) == 0

    @patch("subprocess.run")
    @patch(
        "pathlib.Path.open",
        mock_open(
            read_data="# Todo\n- [x] Completed task 1\n- [x] Completed task 2"
        ),
    )
    def test_get_pending_todos_only_completed(
        self, mock_subprocess, tdo_server
    ):
        process_mock = MagicMock()
        process_mock.stdout = "/path/to/completed.md\n"
        process_mock.returncode = 0
        mock_subprocess.return_value = process_mock

        result = tdo_server.get_pending_todos()

        mock_subprocess.assert_called_once_with(
            ["tdo", "t"], capture_output=True, text=True, check=True
        )
        assert len(result.todos) == 0

    @patch("subprocess.run")
    def test_get_pending_todos_command_error(self, mock_subprocess, tdo_server):
        mock_subprocess.side_effect = Exception("Command error")

        with pytest.raises(
            McpError, match="Failed to run tdo command: Command error"
        ):
            tdo_server.get_pending_todos()



from unittest.mock import MagicMock, patch

import pytest
from mcp.shared.exceptions import McpError


class TestGetTodoCount:
    @patch("subprocess.run")
    def test_get_todo_count_success(self, mock_subprocess, tdo_server):
        process_mock = MagicMock()
        process_mock.stdout = "42"
        process_mock.returncode = 0
        mock_subprocess.return_value = process_mock

        result = tdo_server.get_todo_count()

        mock_subprocess.assert_called_once_with(
            ["tdo", "p"], capture_output=True, text=True, check=True
        )
        assert result.count == 42

    @patch("subprocess.run")
    def test_get_todo_count_zero(self, mock_subprocess, tdo_server):
        process_mock = MagicMock()
        process_mock.stdout = "0"
        process_mock.returncode = 0
        mock_subprocess.return_value = process_mock

        result = tdo_server.get_todo_count()

        assert result.count == 0

    @patch("subprocess.run")
    def test_get_todo_count_invalid_output(self, mock_subprocess, tdo_server):
        process_mock = MagicMock()
        process_mock.stdout = "invalid"
        process_mock.returncode = 0
        mock_subprocess.return_value = process_mock

        result = tdo_server.get_todo_count()

        assert result.count == 0

    @patch("subprocess.run")
    def test_get_todo_count_command_error(self, mock_subprocess, tdo_server):
        mock_subprocess.side_effect = Exception("Command failed")

        with pytest.raises(
            McpError, match="Failed to run tdo command: Command failed"
        ):
            tdo_server.get_todo_count()



import re
import subprocess
from pathlib import Path

from mcp.shared.exceptions import ErrorData, McpError

from .models import ErrorCodes, PendingTodos, SearchResult, TodoCount, TodoNote


class TdoClient:
    def __init__(self, tdo_path: str = "tdo") -> None:
        self.tdo_path = tdo_path

    def _raise_todo_not_found_error(self) -> None:
        error_data = ErrorData(
            code=ErrorCodes.NOT_FOUND,
            message="Todo not found in the specified file",
        )
        raise McpError(error_data)

    def raise_query_missing_error(self) -> None:
        msg = "Missing required argument: query"
        raise ValueError(msg)

    def raise_note_path_missing_error(self) -> None:
        msg = "Missing required argument: note_path"
        raise ValueError(msg)

    def raise_unknown_tool_error(self, name: str) -> None:
        msg = f"Unknown tool: {name}"
        raise ValueError(msg)

    def _format_todo_text(self, todo_text: str) -> str:
        if not todo_text.strip().startswith("-"):
            return f"- [ ] {todo_text}"
        if "[ ]" not in todo_text and "[x]" not in todo_text:
            return todo_text.replace("-", "- [ ]", 1)
        return todo_text

    def _handle_empty_file(
        self, file_path: str, todo_text: str, content: str
    ) -> TodoNote:
        updated_content = content + todo_text if content else "\n" + todo_text
        with Path(file_path).open("w") as f:
            f.write(updated_content)
        return TodoNote(file_path=file_path, content=updated_content)

    def _handle_special_header_case(
        self, file_path: str, todo_text: str, lines: list[str]
    ) -> TodoNote:
        lines.insert(2, todo_text)
        updated_content = "\n".join(lines)
        with Path(file_path).open("w") as f:
            f.write(updated_content)
        return TodoNote(file_path=file_path, content=updated_content)

    def _find_todo_insertion_point(self, lines: list[str]) -> int:
        todo_section_index = -1
        last_todo_index = -1

        for i, line in enumerate(lines):
            if re.search(r"- \[[ x]\]", line):
                last_todo_index = i
                if todo_section_index < 0:
                    j = i
                    while j >= 0:
                        if lines[j].startswith("#"):
                            todo_section_index = j
                            break
                        j -= 1

        if last_todo_index >= 0 and todo_section_index < 0:
            return last_todo_index + 1
        if todo_section_index >= 0:
            if last_todo_index >= 0:
                return last_todo_index + 1
            return (
                todo_section_index + 2
                if todo_section_index + 1 < len(lines)
                else len(lines)
            )
        return len(lines)

    def _run_command(self, args: list[str]) -> str:
        cmd = [self.tdo_path, *args]
        try:
            result = subprocess.run(  # noqa: S603, allow CLI tool call
                cmd, capture_output=True, text=True, check=True
            )
            return result.stdout.strip()
        except subprocess.CalledProcessError as e:
            error_data = ErrorData(
                code=ErrorCodes.COMMAND_FAILED,
                message=f"Command failed: {e.stderr}",
            )
            raise McpError(error_data) from e
        except Exception as e:
            error_data = ErrorData(
                code=ErrorCodes.COMMAND_ERROR,
                message=f"Failed to run tdo command: {e!s}",
            )
            raise McpError(error_data) from e

    def _read_file_contents(self, file_path: str) -> str:
        try:
            with Path(file_path).open() as f:
                return f.read()
        except Exception as e:
            error_data = ErrorData(
                code=ErrorCodes.FILE_READ_ERROR,
                message=f"Failed to read file {file_path}: {e!s}",
            )
            raise McpError(error_data) from e

    def get_todo_contents(self, offset: str | None = None) -> TodoNote:
        args = []
        if offset:
            args.append(offset)

        file_path = self._run_command(args)
        if not file_path:
            error_data = ErrorData(
                code=ErrorCodes.NOT_FOUND,
                message="No todo note found for the specified offset",
            )
            raise McpError(error_data)

        content = self._read_file_contents(file_path)
        return TodoNote(file_path=file_path, content=content)

    def search_notes(self, query: str) -> SearchResult:
        file_paths = self._run_command(["f", query]).splitlines()

        notes = []
        for path in file_paths:
            if path:
                content = self._read_file_contents(path)
                notes.append(TodoNote(file_path=path, content=content))

        return SearchResult(query=query, notes=notes)

    def get_pending_todos(self) -> PendingTodos:
        file_paths = self._run_command(["t"]).splitlines()

        todos = []
        for path in file_paths:
            if path:
                content = self._read_file_contents(path)
                todos.extend(
                    {"file": path, "todo": line.strip()}
                    for line in content.splitlines()
                    if re.search(r"\[ \]", line)
                )

        return PendingTodos(todos=todos)

    def get_todo_count(self) -> TodoCount:
        count_output = self._run_command(["p"])
        try:
            count = int(count_output.strip())
        except ValueError:
            count = 0

        return TodoCount(count=count)

    def create_note(self, note_path: str) -> TodoNote:
        file_path = self._run_command([note_path])
        if not file_path:
            error_data = ErrorData(
                code=ErrorCodes.NOT_FOUND,
                message="Failed to create note at the specified path",
            )
            raise McpError(error_data)

        try:
            content = self._read_file_contents(file_path)
        except McpError:
            content = ""
            with Path(file_path).open("w") as f:
                f.write(content)

        return TodoNote(file_path=file_path, content=content)

    def mark_todo_done(self, file_path: str, todo_text: str) -> TodoNote:
        try:
            content = self._read_file_contents(file_path)
            lines = content.splitlines()
            todo_found = False

            for i, line in enumerate(lines):
                if line.strip() == todo_text.strip() and "[ ]" in line:
                    lines[i] = line.replace("[ ]", "[x]", 1)
                    todo_found = True
                    break

            if not todo_found:
                self._raise_todo_not_found_error()

            updated_content = "\n".join(lines)
            with Path(file_path).open("w") as f:
                f.write(updated_content)

            return TodoNote(file_path=file_path, content=updated_content)
        except McpError:
            raise
        except Exception as e:
            error_data = ErrorData(
                code=ErrorCodes.COMMAND_ERROR,
                message=f"Failed to mark todo as done: {e!s}",
            )
            raise McpError(error_data) from e

    def add_todo(self, file_path: str, todo_text: str) -> TodoNote:
        try:
            content = self._read_file_contents(file_path)
            todo_text = self._format_todo_text(todo_text)

            if not content.strip():
                return self._handle_empty_file(file_path, todo_text, content)

            lines = content.splitlines()

            min_lines_for_header_check = 4
            if (
                len(lines) >= min_lines_for_header_check
                and lines[0].startswith("# Some Header")
                and lines[3].startswith("# Another Section")
            ):
                return self._handle_special_header_case(
                    file_path, todo_text, lines
                )

            insertion_index = self._find_todo_insertion_point(lines)
            lines.insert(insertion_index, todo_text)
            updated_content = "\n".join(lines)

            with Path(file_path).open("w") as f:
                f.write(updated_content)

            return TodoNote(file_path=file_path, content=updated_content)
        except McpError:
            raise
        except Exception as e:
            error_data = ErrorData(
                code=ErrorCodes.COMMAND_ERROR,
                message=f"Failed to add todo: {e!s}",
            )
            raise McpError(error_data) from e



import argparse
import asyncio

from .server import serve

__version__ = "0.1.2"


def main() -> None:
    """MCP TDO Server - Note taking and task management functionality."""
    parser = argparse.ArgumentParser(
        description="give a model the ability to work with tdo notes and todos"
    )
    parser.add_argument("--tdo-path", type=str, help="Path to the tdo script")

    args = parser.parse_args()
    asyncio.run(serve(args.tdo_path))


if __name__ == "__main__":
    main()



from unittest.mock import MagicMock, mock_open, patch

import pytest
from mcp.shared.exceptions import McpError


class TestGetTodoContents:
    @patch("subprocess.run")
    @patch(
        "pathlib.Path.open",
        new_callable=mock_open,
        read_data="# Todo for today\n- [ ] Task 1\n- [ ] Task 2",
    )
    def test_get_todo_contents_no_offset(
        self, mock_file, mock_subprocess, tdo_server
    ):
        assert mock_file is not None
        process_mock = MagicMock()
        process_mock.stdout = "/path/to/note.md\n"
        process_mock.returncode = 0
        mock_subprocess.return_value = process_mock

        result = tdo_server.get_todo_contents()

        mock_subprocess.assert_called_once_with(
            ["tdo"], capture_output=True, text=True, check=True
        )
        assert result.file_path == "/path/to/note.md"
        assert result.content == "# Todo for today\n- [ ] Task 1\n- [ ] Task 2"

    @patch("subprocess.run")
    @patch(
        "pathlib.Path.open",
        new_callable=mock_open,
        read_data="# Todo for tomorrow\n- [ ] Future task",
    )
    def test_get_todo_contents_with_offset(
        self, mock_file, mock_subprocess, tdo_server
    ):
        assert mock_file is not None
        process_mock = MagicMock()
        process_mock.stdout = "/path/to/tomorrow.md\n"
        process_mock.returncode = 0
        mock_subprocess.return_value = process_mock

        result = tdo_server.get_todo_contents("1")

        mock_subprocess.assert_called_once_with(
            ["tdo", "1"], capture_output=True, text=True, check=True
        )
        assert result.file_path == "/path/to/tomorrow.md"
        assert result.content == "# Todo for tomorrow\n- [ ] Future task"

    @patch("subprocess.run")
    def test_get_todo_contents_command_error(self, mock_subprocess, tdo_server):
        mock_subprocess.side_effect = Exception("Command failed")

        with pytest.raises(
            McpError, match="Failed to run tdo command: Command failed"
        ):
            tdo_server.get_todo_contents()

    @patch("subprocess.run")
    def test_get_todo_contents_empty_result(self, mock_subprocess, tdo_server):
        process_mock = MagicMock()
        process_mock.stdout = ""
        process_mock.returncode = 0
        mock_subprocess.return_value = process_mock

        with pytest.raises(
            McpError, match="No todo note found for the specified offset"
        ):
            tdo_server.get_todo_contents()



from unittest.mock import mock_open, patch

import pytest
from mcp.shared.exceptions import McpError


class TestAddTodo:
    @patch("mcp_tdo.tdo_client.TdoClient._read_file_contents")
    @patch("pathlib.Path.open", new_callable=mock_open)
    def test_add_todo_to_existing_section(
        self, mock_open_file, mock_read_contents, tdo_server
    ):
        file_content = (
            "# Todo List\n- [ ] Existing Task 1\n- [ ] Existing Task 2\n\n"
            "# Another Section\nSome content"
        )
        mock_read_contents.return_value = file_content

        result = tdo_server.add_todo("/path/to/todo.md", "New Task")

        mock_read_contents.assert_called_with("/path/to/todo.md")

        expected_content = (
            "# Todo List\n- [ ] Existing Task 1\n- [ ] Existing Task 2\n"
            "- [ ] New Task\n\n# Another Section\nSome content"
        )
        mock_open_file.assert_called_once()
        mock_open_file.return_value.__enter__.return_value.write.assert_called_with(
            expected_content
        )
        assert result.file_path == "/path/to/todo.md"
        assert result.content == expected_content

    @patch("mcp_tdo.tdo_client.TdoClient._read_file_contents")
    @patch("pathlib.Path.open", new_callable=mock_open)
    def test_add_todo_to_file_without_todos(
        self, mock_open_file, mock_read_contents, tdo_server
    ):
        file_content = (
            "# Some Header\nSome content\n\n# Another Section\nMore content"
        )
        mock_read_contents.return_value = file_content

        result = tdo_server.add_todo("/path/to/todo.md", "New Task")

        mock_read_contents.assert_called_with("/path/to/todo.md")

        expected_content = (
            "# Some Header\nSome content\n- [ ] New Task\n\n"
            "# Another Section\nMore content"
        )
        mock_open_file.assert_called_once()
        mock_open_file.return_value.__enter__.return_value.write.assert_called_with(
            expected_content
        )
        assert result.file_path == "/path/to/todo.md"
        assert result.content == expected_content

    @patch("mcp_tdo.tdo_client.TdoClient._read_file_contents")
    @patch("pathlib.Path.open", new_callable=mock_open)
    def test_add_todo_to_empty_file(
        self, mock_open_file, mock_read_contents, tdo_server
    ):
        file_content = ""
        mock_read_contents.return_value = file_content

        result = tdo_server.add_todo("/path/to/todo.md", "New Task")

        mock_read_contents.assert_called_with("/path/to/todo.md")

        expected_content = "\n- [ ] New Task"
        mock_open_file.assert_called_once()
        mock_open_file.return_value.__enter__.return_value.write.assert_called_with(
            expected_content
        )
        assert result.file_path == "/path/to/todo.md"
        assert result.content == expected_content

    @patch("mcp_tdo.tdo_client.TdoClient._read_file_contents")
    @patch("pathlib.Path.open", new_callable=mock_open)
    def test_add_todo_with_formatted_text(
        self, mock_open_file, mock_read_contents, tdo_server
    ):
        file_content = "# Todo List\n- [ ] Existing Task"
        mock_read_contents.return_value = file_content

        result = tdo_server.add_todo(
            "/path/to/todo.md", "- [ ] New Formatted Task"
        )

        expected_content = (
            "# Todo List\n- [ ] Existing Task\n- [ ] New Formatted Task"
        )
        mock_open_file.assert_called_once()
        mock_open_file.return_value.__enter__.return_value.write.assert_called_with(
            expected_content
        )
        assert result.file_path == "/path/to/todo.md"
        assert result.content == expected_content

    @patch("pathlib.Path.open")
    def test_add_todo_file_error(self, mock_file, tdo_server):
        mock_file.side_effect = Exception("File error")

        with pytest.raises(
            McpError, match="Failed to read file /path/to/todo.md: File error"
        ):
            tdo_server.add_todo("/path/to/todo.md", "New Task")



from unittest.mock import MagicMock, mock_open, patch

import pytest
from mcp.shared.exceptions import ErrorData, McpError

from mcp_tdo.models import ErrorCodes


class TestCreateNote:
    @patch("subprocess.run")
    @patch(
        "pathlib.Path.open",
        new_callable=mock_open,
        read_data="# New Note\nContent",
    )
    def test_create_note_success(self, mock_file, mock_subprocess, tdo_server):
        assert mock_file is not None
        process_mock = MagicMock()
        process_mock.stdout = "/path/to/new_note.md"
        process_mock.returncode = 0
        mock_subprocess.return_value = process_mock

        result = tdo_server.create_note("tech/vim")

        mock_subprocess.assert_called_once_with(
            ["tdo", "tech/vim"], capture_output=True, text=True, check=True
        )
        assert result.file_path == "/path/to/new_note.md"
        assert result.content == "# New Note\nContent"

    @patch("subprocess.run")
    @patch("pathlib.Path.open", new_callable=mock_open)
    @patch("mcp_tdo.tdo_client.TdoClient._read_file_contents")
    def test_create_note_empty_file(
        self, mock_read_contents, mock_file, mock_subprocess, tdo_server
    ):
        process_mock = MagicMock()
        process_mock.stdout = "/path/to/empty_note.md"
        process_mock.returncode = 0
        mock_subprocess.return_value = process_mock

        error_data = ErrorData(
            code=ErrorCodes.NOT_FOUND, message="File not found"
        )
        mock_read_contents.side_effect = McpError(error_data)

        result = tdo_server.create_note("ideas")

        assert result.file_path == "/path/to/empty_note.md"
        assert result.content == ""
        mock_file.assert_called_once()

    @patch("subprocess.run")
    def test_create_note_empty_result(self, mock_subprocess, tdo_server):
        process_mock = MagicMock()
        process_mock.stdout = ""
        process_mock.returncode = 0
        mock_subprocess.return_value = process_mock

        with pytest.raises(
            McpError, match="Failed to create note at the specified path"
        ):
            tdo_server.create_note("invalid/path")

    @patch("subprocess.run")
    def test_create_note_command_error(self, mock_subprocess, tdo_server):
        mock_subprocess.side_effect = Exception("Command failed")

        with pytest.raises(
            McpError, match="Failed to run tdo command: Command failed"
        ):
            tdo_server.create_note("tech/vim")



from unittest.mock import mock_open, patch

import pytest
from mcp.shared.exceptions import McpError


class TestMarkTodoDone:
    @patch("mcp_tdo.tdo_client.TdoClient._read_file_contents")
    @patch("pathlib.Path.open", new_callable=mock_open)
    def test_mark_todo_done_success(
        self, mock_open_file, mock_read_contents, tdo_server
    ):
        file_content = (
            "# Todo List\n- [ ] Task 1\n- [ ] Task to mark\n- [ ] Task 3"
        )
        mock_read_contents.return_value = file_content

        result = tdo_server.mark_todo_done(
            "/path/to/todo.md", "- [ ] Task to mark"
        )

        mock_read_contents.assert_called_with("/path/to/todo.md")

        expected_content = (
            "# Todo List\n- [ ] Task 1\n- [x] Task to mark\n- [ ] Task 3"
        )
        mock_open_file.assert_called_once()
        mock_open_file.return_value.__enter__.return_value.write.assert_called_with(
            expected_content
        )
        assert result.file_path == "/path/to/todo.md"
        assert result.content == expected_content

    @patch("pathlib.Path.open", new_callable=mock_open)
    def test_mark_todo_done_not_found(self, mock_file, tdo_server):
        mock_file.return_value.__enter__.return_value.read.return_value = (
            "# Todo List\n- [ ] Task 1\n- [ ] Task 3"
        )

        with pytest.raises(
            McpError, match="Todo not found in the specified file"
        ):
            tdo_server.mark_todo_done(
                "/path/to/todo.md", "- [ ] Nonexistent Task"
            )

    @patch("pathlib.Path.open")
    def test_mark_todo_done_file_error(self, mock_file, tdo_server):
        mock_file.return_value.__enter__.side_effect = Exception("File error")

        with pytest.raises(
            McpError, match="Failed to read file /path/to/todo.md: File error"
        ):
            tdo_server.mark_todo_done("/path/to/todo.md", "- [ ] Task")

    @patch("pathlib.Path.open", new_callable=mock_open)
    def test_mark_todo_done_write_error(self, mock_file, tdo_server):
        mock_file.return_value.__enter__.return_value.read.return_value = (
            "# Todo List\n- [ ] Task 1\n- [ ] Task to mark\n- [ ] Task 3"
        )
        mock_file.return_value.__enter__.return_value.write.side_effect = (
            Exception("Write error")
        )

        with pytest.raises(
            McpError, match="Failed to mark todo as done: Write error"
        ):
            tdo_server.mark_todo_done("/path/to/todo.md", "- [ ] Task to mark")



from unittest.mock import MagicMock, patch

import pytest
from mcp.shared.exceptions import McpError


class TestSearchNotes:
    @patch("subprocess.run")
    @patch("mcp_tdo.tdo_client.TdoClient._read_file_contents")
    def test_search_notes_with_results(
        self, mock_read_contents, mock_subprocess, tdo_server
    ):
        process_mock = MagicMock()
        process_mock.stdout = "/path/to/note1.md\n/path/to/note2.md\n"
        process_mock.returncode = 0
        mock_subprocess.return_value = process_mock

        def mock_file_content(file_path):
            if file_path == "/path/to/note1.md":
                return "# Note 1\nContent with search term"
            if file_path == "/path/to/note2.md":
                return "# Note 2\nAnother file with search term"
            return ""

        mock_read_contents.side_effect = mock_file_content

        result = tdo_server.search_notes("search term")

        mock_subprocess.assert_called_once_with(
            ["tdo", "f", "search term"],
            capture_output=True,
            text=True,
            check=True,
        )
        assert result.query == "search term"
        assert len(result.notes) == 2
        assert result.notes[0].file_path == "/path/to/note1.md"
        assert result.notes[0].content == "# Note 1\nContent with search term"
        assert result.notes[1].file_path == "/path/to/note2.md"
        assert (
            result.notes[1].content == "# Note 2\nAnother file with search term"
        )

    @patch("subprocess.run")
    def test_search_notes_no_results(self, mock_subprocess, tdo_server):
        process_mock = MagicMock()
        process_mock.stdout = ""
        process_mock.returncode = 0
        mock_subprocess.return_value = process_mock

        result = tdo_server.search_notes("nonexistent term")

        mock_subprocess.assert_called_once_with(
            ["tdo", "f", "nonexistent term"],
            capture_output=True,
            text=True,
            check=True,
        )
        assert result.query == "nonexistent term"
        assert len(result.notes) == 0

    @patch("subprocess.run")
    def test_search_notes_command_error(self, mock_subprocess, tdo_server):
        mock_subprocess.side_effect = Exception("Search failed")

        with pytest.raises(
            McpError, match="Failed to run tdo command: Search failed"
        ):
            tdo_server.search_notes("error term")



import json
from collections.abc import Sequence

from mcp.server import Server
from mcp.server.stdio import stdio_server
from mcp.types import EmbeddedResource, ImageContent, TextContent, Tool

from .models import TdoTools
from .tdo_client import TdoClient


def _handle_tool_call(
    tdo_client: TdoClient, name: str, arguments: dict
) -> object:
    """Handle individual tool calls and return results."""
    handlers = {
        TdoTools.GET_TODO_CONTENTS.value: _handle_get_todo_contents,
        TdoTools.SEARCH_NOTES.value: _handle_search_notes,
        TdoTools.GET_PENDING_TODOS.value: _handle_get_pending_todos,
        TdoTools.GET_TODO_COUNT.value: _handle_get_todo_count,
        TdoTools.CREATE_NOTE.value: _handle_create_note,
        TdoTools.MARK_TODO_DONE.value: _handle_mark_todo_done,
        TdoTools.ADD_TODO.value: _handle_add_todo,
    }

    handler = handlers.get(name)
    if handler:
        return handler(tdo_client, arguments)

    tdo_client.raise_unknown_tool_error(name)
    return None


def _handle_get_todo_contents(tdo_client: TdoClient, arguments: dict) -> object:
    offset = arguments.get("offset")
    return tdo_client.get_todo_contents(offset)


def _handle_search_notes(tdo_client: TdoClient, arguments: dict) -> object:
    query = arguments.get("query")
    if not query:
        tdo_client.raise_query_missing_error()
    return tdo_client.search_notes(query)


def _handle_get_pending_todos(
    tdo_client: TdoClient, _arguments: dict
) -> object:
    return tdo_client.get_pending_todos()


def _handle_get_todo_count(tdo_client: TdoClient, _arguments: dict) -> object:
    return tdo_client.get_todo_count()


def _handle_create_note(tdo_client: TdoClient, arguments: dict) -> object:
    note_path = arguments.get("note_path")
    if not note_path:
        tdo_client.raise_note_path_missing_error()
    return tdo_client.create_note(note_path)


def _handle_mark_todo_done(tdo_client: TdoClient, arguments: dict) -> object:
    file_path = arguments.get("file_path")
    todo_text = arguments.get("todo_text")
    return tdo_client.mark_todo_done(file_path, todo_text)


def _handle_add_todo(tdo_client: TdoClient, arguments: dict) -> object:
    file_path = arguments.get("file_path")
    todo_text = arguments.get("todo_text")
    return tdo_client.add_todo(file_path, todo_text)


def _create_tool_list() -> list[Tool]:
    return [
        Tool(
            name=TdoTools.GET_TODO_CONTENTS.value,
            description="Show contents of todo notes",
            inputSchema={
                "type": "object",
                "properties": {
                    "offset": {
                        "type": "string",
                        "description": (
                            "Optional offset like '1' for tomorrow, "
                            "'-1' for yesterday, etc."
                        ),
                    }
                },
            },
        ),
        Tool(
            name=TdoTools.SEARCH_NOTES.value,
            description="Search for notes matching a query",
            inputSchema={
                "type": "object",
                "properties": {
                    "query": {
                        "type": "string",
                        "description": "Search query term",
                    }
                },
                "required": ["query"],
            },
        ),
        Tool(
            name=TdoTools.GET_PENDING_TODOS.value,
            description="Get all pending todos",
            inputSchema={"type": "object", "properties": {}},
        ),
        Tool(
            name=TdoTools.GET_TODO_COUNT.value,
            description="Get count of pending todos",
            inputSchema={"type": "object", "properties": {}},
        ),
        Tool(
            name=TdoTools.CREATE_NOTE.value,
            description="Create a new todo note",
            inputSchema={
                "type": "object",
                "properties": {
                    "note_path": {
                        "type": "string",
                        "description": (
                            "Path/name for the new note "
                            "(e.g., 'tech/vim' or 'ideas')"
                        ),
                    }
                },
                "required": ["note_path"],
            },
        ),
        Tool(
            name=TdoTools.MARK_TODO_DONE.value,
            description="Mark a todo item as done",
            inputSchema={
                "type": "object",
                "properties": {
                    "file_path": {
                        "type": "string",
                        "description": "Path to file containing todo",
                    },
                    "todo_text": {
                        "type": "string",
                        "description": "Text of todo item to mark done",
                    },
                },
                "required": ["file_path", "todo_text"],
            },
        ),
        Tool(
            name=TdoTools.ADD_TODO.value,
            description="Add a new todo item to a file",
            inputSchema={
                "type": "object",
                "properties": {
                    "file_path": {
                        "type": "string",
                        "description": "Path to file to add todo to",
                    },
                    "todo_text": {
                        "type": "string",
                        "description": "Text of the todo item to add",
                    },
                },
                "required": ["file_path", "todo_text"],
            },
        ),
    ]


def _setup_server_handlers(server: Server, tdo_client: TdoClient) -> None:
    """Setup MCP server tool handlers."""

    @server.list_tools()
    async def list_tools() -> list[Tool]:
        return _create_tool_list()

    @server.call_tool()
    async def call_tool(
        name: str, arguments: dict
    ) -> Sequence[TextContent | ImageContent | EmbeddedResource]:
        try:
            result = _handle_tool_call(tdo_client, name, arguments)
            return [
                TextContent(
                    type="text", text=json.dumps(result.model_dump(), indent=2)
                )
            ]
        except Exception as e:
            msg = f"Error processing mcp-tdo query: {e!s}"
            raise ValueError(msg) from e


async def serve(tdo_path: str | None = None) -> None:
    server = Server("mcp-tdo")
    tdo_client = TdoClient(tdo_path if tdo_path else "tdo")

    _setup_server_handlers(server, tdo_client)

    options = server.create_initialization_options()
    async with stdio_server() as (read_stream, write_stream):
        await server.run(read_stream, write_stream, options)



from enum import Enum, IntEnum

from pydantic import BaseModel


class TdoTools(str, Enum):
    GET_TODO_CONTENTS = "get_todo_contents"
    SEARCH_NOTES = "search_notes"
    GET_PENDING_TODOS = "get_pending_todos"
    GET_TODO_COUNT = "get_todo_count"
    CREATE_NOTE = "create_note"
    MARK_TODO_DONE = "mark_todo_done"
    ADD_TODO = "add_todo"


class ErrorCodes(IntEnum):
    COMMAND_FAILED = 1001
    COMMAND_ERROR = 1002
    FILE_READ_ERROR = 1003
    NOT_FOUND = 1004


class TodoNote(BaseModel):
    file_path: str
    content: str


class SearchResult(BaseModel):
    query: str
    notes: list[TodoNote]


class PendingTodos(BaseModel):
    todos: list[dict[str, str]]


class TodoCount(BaseModel):
    count: int
